@@ -19,8 +19,8 @@ from trac.perm import IPermissionRequestor
 from trac.ticket.api import ITicketActionController
 from trac.ticket.default_workflow import ConfigurableTicketWorkflow
 
-revision = "$Rev: 13230 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.0-stable/sample-plugins/workflow/CodeReview.py $"
+revision = "$Rev: 13432 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.2.2/sample-plugins/workflow/CodeReview.py $"
 
 class CodeReviewActionController(Component):
     """Support for simple code reviews.
@@ -32,7 +32,7 @@ class CodeReviewActionController(Component):
     Example (from the enterprise-review-workflow.ini):
     {{{
     review = in_review -> *
-    review.name = review as
+    review.label = review as
     review.operations = code_review
     review.code_review =
       approve -> in_QA,
@@ -88,7 +88,7 @@ class CodeReviewActionController(Component):
 
         selected_value = grade or review_options[0][0]
 
-        label = actions[action]['name']
+        label = actions[action]['label']
         control = tag(["as: ",
                        tag.select([tag.option(option, selected=
                                               (option == selected_value or
