@@ -67,7 +67,7 @@ from datetime import datetime, timedelta
 from pkg_resources import parse_version
 
 import trac
-from trac.tests.compat import rmtree
+from trac.test import rmtree
 from trac.util.compat import close_fds
 
 # Handle missing twill so we can print a useful 'SKIP'
@@ -151,6 +151,7 @@ if twill:
             self._testenv.start()
             self._tester = self.tester_class(baseurl)
             self.fixture = (self._testenv, self._tester)
+            self._testenv.set_config('project', 'name', 'Functional Tests')
 
         def tearDown(self):
             self._testenv.stop()
