@@ -15,6 +15,7 @@
 Provides some Trac environment-wide utility functions, and a way to call
 :command:`trac-admin` without it being on the path."""
 
+import hashlib
 import locale
 import os
 import re
@@ -22,20 +23,16 @@ import sys
 import time
 from subprocess import call, Popen, PIPE, STDOUT
 
+from trac.config import UnicodeConfigParser
+from trac.db.api import DatabaseManager
 from trac.env import open_environment
-from trac.test import EnvironmentStub, get_dburi
-from trac.tests.compat import rmtree
-from trac.tests.functional import logfile, trac_source_tree
+from trac.test import EnvironmentStub, get_dburi, rmtree
+from trac.tests.functional import trac_source_tree
 from trac.tests.functional.better_twill import tc, ConnectError
-from trac.util import terminate
+from trac.util import create_file, terminate
 from trac.util.compat import close_fds, wait_for_file_mtime_change
 from trac.util.text import to_utf8
 
-try:
-    from configobj import ConfigObj
-except ImportError:
-    ConfigObj = None
-
 # TODO: refactor to support testing multiple frontends, backends
 #       (and maybe repositories and authentication).
 #
@@ -90,7 +87,6 @@ class FunctionalTestEnvironment(object):
         """Remove all of the test environment data."""
         env = EnvironmentStub(path=self.tracdir, destroying=True)
         env.destroy_db()
-        env.shutdown()
 
         self.destroy_repo()
         if os.path.exists(self.dirname):
@@ -136,6 +132,8 @@ class FunctionalTestEnvironment(object):
         authentication.
         """
         os.mkdir(self.dirname)
+        # testing.log gets any unused output from subprocesses
+        self.logfile = open(os.path.join(self.dirname, 'testing.log'), 'w')
         self.create_repo()
 
         self._tracadmin('initenv', self.tracdir, self.dburi, self.repotype,
@@ -160,6 +158,7 @@ class FunctionalTestEnvironment(object):
         """Add a user to the environment.  The password will be set to the
         same as username."""
         user = to_utf8(user)
+        self._tracadmin('session', 'add', user)
         if call([sys.executable, os.path.join(self.trac_src, 'contrib',
                  'htpasswd.py'), '-b', self.htpasswd,
                  user, user], close_fds=close_fds, cwd=self.command_cwd):
@@ -234,7 +233,7 @@ class FunctionalTestEnvironment(object):
         out = proc.communicate(input=input)[0]
         if proc.returncode:
             print(out)
-            logfile.write(out)
+            self.logfile.write(out)
             raise Exception("Failed while running trac-admin with arguments %r.\n"
                             "Exitcode: %s \n%s"
                             % (args, proc.returncode, out))
@@ -260,7 +259,7 @@ class FunctionalTestEnvironment(object):
         args.append(os.path.join(self.trac_src, 'trac', 'web',
                                  'standalone.py'))
         server = Popen(args + options + [self.tracdir],
-                       stdout=logfile, stderr=logfile,
+                       stdout=self.logfile, stderr=self.logfile,
                        close_fds=close_fds,
                        cwd=self.command_cwd)
         self.pid = server.pid
@@ -283,7 +282,7 @@ class FunctionalTestEnvironment(object):
         FIXME: probably needs a nicer way to exit for coverage to work
         """
         if self.pid:
-            terminate(self)
+            terminate(self.pid)
 
     def restart(self):
         """Restarts the webserver"""
@@ -299,13 +298,13 @@ class FunctionalTestEnvironment(object):
         return "''" # needed for Python 2.3 and 2.4 on win32
 
     def call_in_dir(self, dir, args, environ=None):
-        proc = Popen(args, stdout=PIPE, stderr=logfile,
+        proc = Popen(args, stdout=PIPE, stderr=self.logfile,
                      close_fds=close_fds, cwd=dir, env=environ)
         (data, _) = proc.communicate()
         if proc.wait():
             raise Exception('Unable to run command %s in %s' %
                             (args, dir))
-        logfile.write(data)
+        self.logfile.write(data)
         return data
 
     def enable_authz_permpolicy(self, authz_content, filename=None):
@@ -333,25 +332,29 @@ class FunctionalTestEnvironment(object):
         be generated by computing a hash of `authz_content`, prefixed with
         "authz-".
         """
-        if not ConfigObj:
-            raise ImportError("Can't enable authz permissions policy. " +
-                              "ConfigObj not installed.")
         if filename is None:
-            from hashlib import md5
-            filename = 'authz-' + md5(str(authz_content)).hexdigest()[0:9]
+            filename = 'authz-' + \
+                       hashlib.md5(str(authz_content)).hexdigest()[:9]
         env = self.get_trac_environment()
+        authz_file = os.path.join(env.conf_dir, filename)
+        if os.path.exists(authz_file):
+            wait_for_file_mtime_change(authz_file)
+        if isinstance(authz_content, basestring):
+            authz_content = [line.strip() + '\n'
+                             for line in authz_content.strip().splitlines()]
+            authz_content = ['# -*- coding: utf-8 -*-\n'] + authz_content
+            create_file(authz_file, authz_content)
+        else:
+            parser = UnicodeConfigParser()
+            for section, options in authz_content.items():
+                parser.add_section(section)
+                for key, value in options.items():
+                    parser.set(section, key, value)
+            with open(authz_file, 'w') as f:
+                parser.write(f)
         permission_policies = env.config.get('trac', 'permission_policies')
         env.config.set('trac', 'permission_policies',
                        'AuthzPolicy, ' + permission_policies)
-        authz_file = self.tracdir + '/conf/' + filename
-        if isinstance(authz_content, basestring):
-            authz_content = [line.strip() for line in
-                             authz_content.strip().splitlines()]
-        authz_config = ConfigObj(authz_content, encoding='utf8',
-                                 write_empty_values=True, indent_type='')
-        authz_config.filename = authz_file
-        wait_for_file_mtime_change(authz_file)
-        authz_config.write()
         env.config.set('authz_policy', 'authz_file', authz_file)
         env.config.set('components', 'tracopt.perm.authz_policy.*', 'enabled')
         env.config.save()
