@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006 Edgewall Software
+# Copyright (C)2006-2009 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -17,33 +17,39 @@ import os
 import signal
 import sys
 
+
 def daemonize(pidfile=None, progname=None, stdin='/dev/null',
               stdout='/dev/null', stderr='/dev/null', umask=022):
     """Fork a daemon process."""
-
     if pidfile:
         # Check whether the pid file already exists and refers to a still
         # process running
         pidfile = os.path.abspath(pidfile)
         if os.path.exists(pidfile):
-            fileobj = open(pidfile)
-            try:
+            with open(pidfile) as fileobj:
                 try:
                     pid = int(fileobj.read())
                 except ValueError:
-                    sys.exit('Invalid PID in file %s' % pidfile)
-            finally:
-                fileobj.close()
+                    sys.exit('Invalid pid in file %s\nPlease remove it to '
+                             'proceed' % pidfile)
 
             try: # signal the process to see if it is still running
                 os.kill(pid, 0)
                 if not progname:
                     progname = os.path.basename(sys.argv[0])
                 sys.exit('%s is already running with pid %s' % (progname, pid))
-            except OSError, e:
+            except OSError as e:
                 if e.errno != errno.ESRCH:
                     raise
 
+        # The pid file must be writable
+        try:
+            fileobj = open(pidfile, 'a+')
+            fileobj.close()
+        except IOError as e:
+            from trac.util.text import exception_to_unicode
+            sys.exit('Error writing to pid file: %s' % exception_to_unicode(e))
+
     # Perform first fork
     pid = os.fork()
     if pid > 0:
@@ -73,18 +79,15 @@ def daemonize(pidfile=None, progname=None, stdin='/dev/null',
         # Register signal handlers to ensure atexit hooks are called on exit
         for signum in [signal.SIGTERM, signal.SIGHUP]:
             signal.signal(signum, handle_signal)
-        
+
         # Create/update the pid file, and register a hook to remove it when the
         # process exits
         def remove_pidfile():
             if os.path.exists(pidfile):
-                   os.remove(pidfile)
+                os.remove(pidfile)
         atexit.register(remove_pidfile)
-        fileobj = open(pidfile, 'w')
-        try:
+        with open(pidfile, 'w') as fileobj:
             fileobj.write(str(os.getpid()))
-        finally:
-            fileobj.close()
 
 
 def handle_signal(signum, frame):
