@@ -11,21 +11,18 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
-from __future__ import with_statement
-
 import doctest
 import os.path
 import pkg_resources
 import random
 import re
-import shutil
 import sys
 import tempfile
 import unittest
 
 import trac
-import trac.tests.compat
 from trac import util
+from trac.test import rmtree
 from trac.util.tests import concurrency, datefmt, presentation, text, \
                             translation, html
 
@@ -37,7 +34,7 @@ class AtomicFileTestCase(unittest.TestCase):
         self.path = os.path.join(self.dir, 'trac-tempfile')
 
     def tearDown(self):
-        shutil.rmtree(self.dir)
+        rmtree(self.dir)
 
     def test_non_existing(self):
         with util.AtomicFile(self.path) as f:
@@ -178,7 +175,7 @@ class SafeReprTestCase(unittest.TestCase):
         self.assertRaises(Exception, repr, eh_ix())
         sr = util.safe_repr(eh_ix())
         sr = re.sub('[A-F0-9]{4,}', 'ADDRESS', sr)
-        sr = re.sub(r'__main__|trac\.util\.tests', 'MODULE', sr)
+        sr = re.sub(r'__main__|trac\.util\.tests(\.__init__)?', 'MODULE', sr)
         self.assertEqual("<MODULE.eh_ix object at 0xADDRESS "
                          "(repr() error: TypeError: unsupported operand "
                          "type(s) for +: 'int' and 'str')>", sr)
@@ -186,6 +183,14 @@ class SafeReprTestCase(unittest.TestCase):
 
 class SetuptoolsUtilsTestCase(unittest.TestCase):
 
+    def setUp(self):
+        self.dir = tempfile.mkdtemp()
+        sys.path.append(self.dir)
+
+    def tearDown(self):
+        sys.path.remove(self.dir)
+        rmtree(self.dir)
+
     def test_get_module_path(self):
         self.assertEqual(util.get_module_path(trac),
                          util.get_module_path(util))
@@ -255,6 +260,37 @@ class SetuptoolsUtilsTestCase(unittest.TestCase):
             self.assertNotEqual({}, pkginfo)
             self.assertEqual(pkginfo, util.get_pkginfo(psycopg2.extensions))
 
+    def test_file_metadata(self):
+        pkgname = 'TestModule_' + util.hex_entropy(16)
+        modname = pkgname.lower()
+        with open(os.path.join(self.dir, pkgname + '-0.1.egg-info'), 'w') as f:
+            f.write('Metadata-Version: 1.1\n'
+                    'Name: %(pkgname)s\n'
+                    'Version: 0.1\n'
+                    'Author: Joe\n'
+                    'Author-email: joe@example.org\n'
+                    'Home-page: http://example.org/\n'
+                    'Summary: summary.\n'
+                    'Description: description.\n'
+                    'Provides: %(modname)s\n'
+                    'Provides: %(modname)s.foo\n'
+                    % {'pkgname': pkgname, 'modname': modname})
+        os.mkdir(os.path.join(self.dir, modname))
+        for name in ('__init__.py', 'bar.py', 'foo.py'):
+            with open(os.path.join(self.dir, modname, name), 'w') as f:
+                f.write('# -*- coding: utf-8 -*-\n')
+
+        mod = __import__(modname, {}, {}, ['bar', 'foo'])
+        pkginfo = util.get_pkginfo(mod)
+        self.assertEqual('0.1', pkginfo['version'])
+        self.assertEqual('Joe', pkginfo['author'])
+        self.assertEqual('joe@example.org', pkginfo['author_email'])
+        self.assertEqual('http://example.org/', pkginfo['home_page'])
+        self.assertEqual('summary.', pkginfo['summary'])
+        self.assertEqual('description.', pkginfo['description'])
+        self.assertEqual(pkginfo, util.get_pkginfo(mod.bar))
+        self.assertEqual(pkginfo, util.get_pkginfo(mod.foo))
+
 
 class LazyClass(object):
     @util.lazy
@@ -289,7 +325,7 @@ class FileTestCase(unittest.TestCase):
         self.data = 'Lorem\ripsum\ndolor\r\nsit\namet,\rconsectetur\r\n'
 
     def tearDown(self):
-        shutil.rmtree(self.dir)
+        rmtree(self.dir)
 
     def test_create_and_read_file(self):
         util.create_file(self.filename, self.data, 'wb')
@@ -309,7 +345,7 @@ class FileTestCase(unittest.TestCase):
         self.assertEqual(0, os.path.getsize(self.filename))
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(AtomicFileTestCase))
     suite.addTest(unittest.makeSuite(PathTestCase))
@@ -319,14 +355,14 @@ def suite():
     suite.addTest(unittest.makeSuite(SetuptoolsUtilsTestCase))
     suite.addTest(unittest.makeSuite(LazyTestCase))
     suite.addTest(unittest.makeSuite(FileTestCase))
-    suite.addTest(concurrency.suite())
-    suite.addTest(datefmt.suite())
-    suite.addTest(presentation.suite())
+    suite.addTest(concurrency.test_suite())
+    suite.addTest(datefmt.test_suite())
+    suite.addTest(presentation.test_suite())
     suite.addTest(doctest.DocTestSuite(util))
-    suite.addTest(text.suite())
-    suite.addTest(translation.suite())
-    suite.addTest(html.suite())
+    suite.addTest(text.test_suite())
+    suite.addTest(translation.test_suite())
+    suite.addTest(html.test_suite())
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
