@@ -1,58 +1,98 @@
 # -*- coding: utf-8 -*-
+#
+# Copyright (C) 2006-2013 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/log/.
 
+import os
+import socket
 import unittest
 from StringIO import StringIO
 
-from trac.util.text import empty, expandtabs, fix_eol, javascript_quote, \
-                           to_js_string, normalize_whitespace, to_unicode, \
-                           text_width, print_table, unicode_quote, \
-                           unicode_quote_plus, unicode_unquote, \
-                           unicode_urlencode, wrap, quote_query_string, \
-                           unicode_to_base64, unicode_from_base64, \
-                           levenshtein_distance
+import trac.tests.compat
+from trac.util.text import (
+    _get_default_ambiwidth, empty, expandtabs, fix_eol, javascript_quote,
+    levenshtein_distance, normalize_whitespace, print_table,
+    quote_query_string, shorten_line, strip_line_ws, stripws, sub_vars,
+    text_width, to_js_string, to_unicode, to_utf8, unicode_from_base64,
+    unicode_quote, unicode_quote_plus, unicode_to_base64, unicode_unquote,
+    unicode_urlencode, wrap)
 
 
 class ToUnicodeTestCase(unittest.TestCase):
     def test_explicit_charset(self):
         uc = to_unicode('\xc3\xa7', 'utf-8')
-        assert isinstance(uc, unicode)
-        self.assertEquals(u'\xe7', uc)
+        self.assertIsInstance(uc, unicode)
+        self.assertEqual(u'\xe7', uc)
 
     def test_explicit_charset_with_replace(self):
         uc = to_unicode('\xc3', 'utf-8')
-        assert isinstance(uc, unicode)
-        self.assertEquals(u'\xc3', uc)
+        self.assertIsInstance(uc, unicode)
+        self.assertEqual(u'\xc3', uc)
 
     def test_implicit_charset(self):
         uc = to_unicode('\xc3\xa7')
-        assert isinstance(uc, unicode)
-        self.assertEquals(u'\xe7', uc)
+        self.assertIsInstance(uc, unicode)
+        self.assertEqual(u'\xe7', uc)
 
     def test_from_exception_using_unicode_args(self):
         u = u'\uB144'
         try:
             raise ValueError, '%s is not a number.' % u
-        except ValueError, e:
-            self.assertEquals(u'\uB144 is not a number.', to_unicode(e))
+        except ValueError as e:
+            self.assertEqual(u'\uB144 is not a number.', to_unicode(e))
 
     def test_from_exception_using_str_args(self):
         u = u'Das Ger\xe4t oder die Ressource ist belegt'
         try:
             raise ValueError, u.encode('utf-8')
-        except ValueError, e:
-            self.assertEquals(u, to_unicode(e))
+        except ValueError as e:
+            self.assertEqual(u, to_unicode(e))
+
+    def test_from_windows_error(self):
+        try:
+            os.stat('non/existent/file.txt')
+        except OSError as e:
+            uc = to_unicode(e)
+            self.assertIsInstance(uc, unicode, uc)
+            self.assertTrue(uc.startswith('[Error '), uc)
+            self.assertIn(e.strerror.decode('mbcs'), uc)
+
+    def test_from_socket_error(self):
+        for res in socket.getaddrinfo('127.0.0.1', 65536, 0,
+                                      socket.SOCK_STREAM):
+            af, socktype, proto, canonname, sa = res
+            s = socket.socket(af, socktype, proto)
+            try:
+                s.connect(sa)
+            except socket.error as e:
+                uc = to_unicode(e)
+                self.assertIsInstance(uc, unicode, uc)
+                if hasattr(e, 'strerror'):
+                    self.assertIn(e.strerror.decode('mbcs'), uc)
+
+    if os.name != 'nt':
+        del test_from_windows_error
+        del test_from_socket_error
 
 
 class ExpandtabsTestCase(unittest.TestCase):
     def test_empty(self):
         x = expandtabs('', ignoring='\0')
-        self.assertEquals('', x)
+        self.assertEqual('', x)
     def test_ingoring(self):
         x = expandtabs('\0\t', ignoring='\0')
-        self.assertEquals('\0        ', x)
+        self.assertEqual('\0        ', x)
     def test_tabstops(self):
-        self.assertEquals('        ', expandtabs('       \t'))
-        self.assertEquals('                ', expandtabs('\t\t'))
+        self.assertEqual('        ', expandtabs('       \t'))
+        self.assertEqual('                ', expandtabs('\t\t'))
 
 
 class JavascriptQuoteTestCase(unittest.TestCase):
@@ -120,6 +160,31 @@ class QuoteQueryStringTestCase(unittest.TestCase):
                          quote_query_string(text))
 
 
+class ToUtf8TestCase(unittest.TestCase):
+    def test_unicode(self):
+        self.assertEqual('à', to_utf8('à'))
+        self.assertEqual('ç', to_utf8('ç'))
+
+    def test_boolean(self):
+        self.assertEqual('True', to_utf8(True))
+        self.assertEqual('False', to_utf8(False))
+
+    def test_int(self):
+        self.assertEqual('-1', to_utf8(-1))
+        self.assertEqual('0', to_utf8(0))
+        self.assertEqual('1', to_utf8(1))
+
+    def test_utf8(self):
+        self.assertEqual('à', to_utf8(u'à'))
+        self.assertEqual('ç', to_utf8(u'ç'))
+
+    def test_exception_with_utf8_message(self):
+        self.assertEqual('thė mèssägē', to_utf8(Exception('thė mèssägē')))
+
+    def test_exception_with_unicode_message(self):
+        self.assertEqual('thė mèssägē', to_utf8(Exception(u'thė mèssägē')))
+
+
 class WhitespaceTestCase(unittest.TestCase):
     def test_default(self):
         self.assertEqual(u'This is text ',
@@ -162,15 +227,15 @@ class PrintTableTestCase(unittest.TestCase):
         headers = ('Version', 'Date', 'Name')
         expected = """\
 
-Version     Date         Name     
+Version     Date         Name
 ----------------------------------
-Trac 0.12 | 2010-06-13 | Babel    
-Trac 0.11 | 2008-06-22 | Genshi   
-Trac 0.10 | 2006-09-28 | Zengia   
-Trac 0.9  | 2005-10-31 | Vodun    
-Trac 0.8  | 2004-11-15 | Qualia   
-Trac 0.7  | 2004-05-18 | Fulci    
-Trac 0.6  | 2004-03-23 | Solanum  
+Trac 0.12 | 2010-06-13 | Babel
+Trac 0.11 | 2008-06-22 | Genshi
+Trac 0.10 | 2006-09-28 | Zengia
+Trac 0.9  | 2005-10-31 | Vodun
+Trac 0.8  | 2004-11-15 | Qualia
+Trac 0.7  | 2004-05-18 | Fulci
+Trac 0.6  | 2004-03-23 | Solanum
 Trac 0.5  | 2004-02-23 | Incognito
 
 """
@@ -187,11 +252,11 @@ Trac 0.5  | 2004-02-23 | Incognito
         )
         expected = u"""\
 
-NoneType | None  |      
-bool     | True  | True 
+NoneType | None  |
+bool     | True  | True
 bool     | False | False
-int      | 0     | 0    
-float    | 0.0   | 0.0  
+int      | 0     | 0
+float    | 0.0   | 0.0
 
 """
         self._validate_print_table(expected, data, sep=' | ', ambiwidth=1)
@@ -204,9 +269,9 @@ float    | 0.0   | 0.0
         headers = ('Obfuscated', 'Email')
         expected = u"""\
 
-Obfuscated      Email          
+Obfuscated      Email
 -------------------------------
-foo@localhost | foo@localhost  
+foo@localhost | foo@localhost
 bar@….com     | bar@example.com
 
 """
@@ -221,20 +286,42 @@ bar@….com     | bar@example.com
         headers = ('Obfuscated', 'Email')
         expected = u"""\
 
-Obfuscated      Email          
+Obfuscated      Email
 -------------------------------
-foo@localhost | foo@localhost  
+foo@localhost | foo@localhost
 bar@….com    | bar@example.com
 
 """
         self._validate_print_table(expected, data, headers=headers, sep=' | ',
                                    ambiwidth=2)
 
+    def test_multilines_in_cell(self):
+        data = (
+            (41, 'Trac', 'Trac-Hacks'),
+            (42, 'blah', 'foo\r\nbar\r\n'),
+            (43, 'alfa\r\nbravo\r\n', 'zero\r\none\r\ntwo'),
+        )
+        headers = ('Id', 'Column 1', 'Column 2')
+        expected = u"""\
+
+Id   Column 1   Column 2
+--------------------------
+41 | Trac     | Trac-Hacks
+42 | blah     | foo
+   |          | bar
+43 | alfa     | zero
+   | bravo    | one
+   |          | two
+
+"""
+        self._validate_print_table(expected, data, headers=headers, sep=' | ')
+
     def _validate_print_table(self, expected, data, **kwargs):
         out = StringIO()
         kwargs['out'] = out
         print_table(data, **kwargs)
-        self.assertEqual(expected.encode('utf-8'), out.getvalue())
+        self.assertEqual(expected.encode('utf-8'),
+                         strip_line_ws(out.getvalue(), leading=False))
 
 
 class WrapTestCase(unittest.TestCase):
@@ -311,6 +398,20 @@ class UnicodeBase64TestCase(unittest.TestCase):
         self.assertEqual(text, unicode_from_base64(text_base64_no_strip))
 
 
+class StripwsTestCase(unittest.TestCase):
+    def test_stripws(self):
+        self.assertEqual(u'stripws',
+                         stripws(u' \u200b\t\u3000stripws \u200b\t\u2008'))
+        self.assertEqual(u'stripws \u3000\t',
+                         stripws(u'\u200b\t\u2008 stripws \u3000\t',
+                                 trailing=False))
+        self.assertEqual(u' \t\u3000stripws',
+                         stripws(u' \t\u3000stripws \u200b\t\u2008',
+                                 leading=False))
+        self.assertEqual(u' \t\u3000stripws \u200b\t\u2008',
+                         stripws(u' \t\u3000stripws \u200b\t\u2008',
+                                 leading=False, trailing=False))
+
 
 class LevenshteinDistanceTestCase(unittest.TestCase):
     def test_distance(self):
@@ -321,22 +422,119 @@ class LevenshteinDistanceTestCase(unittest.TestCase):
         self.assertEqual(0, levenshtein_distance('milestone', 'milestone'))
 
 
-def suite():
+class SubVarsTestCase(unittest.TestCase):
+    def test_sub_vars(self):
+        subtext = sub_vars("$USER's tickets for '$COMPONENT', $MILESTONE",
+                           {'USER': 'user1', 'COMPONENT': 'component1'})
+        self.assertEqual("user1's tickets for 'component1', $MILESTONE",
+                         subtext)
+
+
+class ShortenLineTestCase(unittest.TestCase):
+
+    def test_less_than_maxlen(self):
+        text = '123456789'
+        self.assertEqual(text, shorten_line(text, 10))
+
+    def test_equalto_maxlen(self):
+        text = '1234567890'
+        self.assertEqual(text, shorten_line(text, 10))
+
+    def test_greater_than_maxlen(self):
+        text = 'word word word word'
+        self.assertEqual('word word ...', shorten_line(text, 15))
+        text = 'abcdefghij'
+        self.assertEqual('abcde ...', shorten_line(text, 9))
+
+
+class DefaultAmbiwidthTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.environ = os.environ.copy()
+
+    def tearDown(self):
+        for name in set(os.environ) - set(self.environ):
+            del os.environ[name]
+        os.environ.update(self.environ)
+
+    def _unset_locale_envs(self):
+        for name in ('LANGUAGE', 'LC_ALL', 'LC_MESSAGES', 'LANG'):
+            if name in os.environ:
+                del os.environ[name]
+
+    def _test_ambiwidth(self, expected, envs):
+        self._unset_locale_envs()
+        os.environ.update(envs)
+        self.assertEqual(expected, _get_default_ambiwidth())
+
+    def test_no_locale_envs(self):
+        self._unset_locale_envs()
+        self.assertEqual(1, _get_default_ambiwidth())
+
+    def test_language(self):
+        self._test_ambiwidth(1, {'LANGUAGE': 'C'})
+        self._test_ambiwidth(1, {'LANGUAGE': 'POSIX'})
+        self._test_ambiwidth(1, {'LANGUAGE': 'de_DE'})
+        self._test_ambiwidth(2, {'LANGUAGE': 'ko'})
+        self._test_ambiwidth(2, {'LANGUAGE': 'ko_KR'})
+        self._test_ambiwidth(2, {'LANGUAGE': 'ja'})
+        self._test_ambiwidth(2, {'LANGUAGE': 'ja_JP'})
+        self._test_ambiwidth(2, {'LANGUAGE': 'zh_CN'})
+        self._test_ambiwidth(2, {'LANGUAGE': 'zh_TW'})
+        self._test_ambiwidth(1, {'LANGUAGE': 'en_US:ja:zh_TW'})
+        self._test_ambiwidth(2, {'LANGUAGE': 'zh_CN:en:ko'})
+        self._test_ambiwidth(1, {'LANGUAGE': '*****'})
+
+    def test_simple_locale_env(self):
+        for name in ('LC_ALL', 'LC_MESSAGES', 'LANG'):
+            self._test_ambiwidth(1, {name: 'C'})
+            self._test_ambiwidth(1, {name: 'POSIX'})
+            self._test_ambiwidth(1, {name: 'en_US.UTF8'})
+            self._test_ambiwidth(1, {name: 'de_DE.UTF8'})
+            self._test_ambiwidth(2, {name: 'ko_KR.UTF8'})
+            self._test_ambiwidth(2, {name: 'ja_JP.UTF8'})
+            self._test_ambiwidth(2, {name: 'zh_CN.UTF8'})
+            self._test_ambiwidth(2, {name: 'zh_TW.UTF8'})
+            self._test_ambiwidth(1, {name: '*****'})
+
+    def test_combined_locale_envs(self):
+        os.environ.update({'LANGUAGE': 'en_US',
+                           'LC_ALL': 'zh_TW.UTF8', 'LC_MESSAGES': 'de_DE.UTF8',
+                           'LANG': 'ko_KR.UTF8'})
+        self.assertEqual(1, _get_default_ambiwidth())
+        del os.environ['LANGUAGE']
+        self.assertEqual(2, _get_default_ambiwidth())
+        del os.environ['LC_ALL']
+        self.assertEqual(1, _get_default_ambiwidth())
+        del os.environ['LC_MESSAGES']
+        self.assertEqual(2, _get_default_ambiwidth())
+        del os.environ['LANG']
+        self.assertEqual(1, _get_default_ambiwidth())
+
+
+def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(ToUnicodeTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(ExpandtabsTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(UnicodeQuoteTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(JavascriptQuoteTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(ToJsStringTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(QuoteQueryStringTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(WhitespaceTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(TextWidthTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(PrintTableTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(WrapTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(FixEolTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(UnicodeBase64TestCase, 'test'))
-    suite.addTest(unittest.makeSuite(LevenshteinDistanceTestCase, 'test'))
+    suite.addTest(unittest.makeSuite(ToUnicodeTestCase))
+    suite.addTest(unittest.makeSuite(ExpandtabsTestCase))
+    suite.addTest(unittest.makeSuite(UnicodeQuoteTestCase))
+    suite.addTest(unittest.makeSuite(JavascriptQuoteTestCase))
+    suite.addTest(unittest.makeSuite(ToJsStringTestCase))
+    suite.addTest(unittest.makeSuite(QuoteQueryStringTestCase))
+    suite.addTest(unittest.makeSuite(ToUtf8TestCase))
+    suite.addTest(unittest.makeSuite(WhitespaceTestCase))
+    suite.addTest(unittest.makeSuite(TextWidthTestCase))
+    suite.addTest(unittest.makeSuite(PrintTableTestCase))
+    suite.addTest(unittest.makeSuite(WrapTestCase))
+    suite.addTest(unittest.makeSuite(FixEolTestCase))
+    suite.addTest(unittest.makeSuite(UnicodeBase64TestCase))
+    suite.addTest(unittest.makeSuite(StripwsTestCase))
+    suite.addTest(unittest.makeSuite(LevenshteinDistanceTestCase))
+    suite.addTest(unittest.makeSuite(SubVarsTestCase))
+    suite.addTest(unittest.makeSuite(ShortenLineTestCase))
+    if os.name != 'nt':
+        suite.addTest(unittest.makeSuite(DefaultAmbiwidthTestCase))
     return suite
 
+
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
