@@ -19,9 +19,9 @@ from cStringIO import StringIO
 from datetime import datetime, timedelta
 from subprocess import Popen, PIPE
 
+import trac.tests.compat
 from trac.core import TracError
-from trac.test import EnvironmentStub, Mock, MockPerm, locate
-from trac.tests.compat import rmtree
+from trac.test import EnvironmentStub, MockRequest, locate, rmtree
 from trac.util import create_file
 from trac.util.compat import close_fds
 from trac.util.datefmt import to_timestamp, utc
@@ -32,10 +32,10 @@ from trac.versioncontrol.api import Changeset, DbRepositoryProvider, \
                                     RepositoryManager
 from trac.versioncontrol.web_ui.browser import BrowserModule
 from trac.versioncontrol.web_ui.log import LogModule
-from trac.web.href import Href
 from tracopt.versioncontrol.git.PyGIT import StorageFactory
 from tracopt.versioncontrol.git.git_fs import GitCachedRepository, \
-                                              GitConnector, GitRepository
+                                              GitRepository, \
+                                              GitwebProjectsRepositoryProvider
 
 
 class GitCommandMixin(object):
@@ -61,6 +61,7 @@ class GitCommandMixin(object):
     def _git(self, *args, **kwargs):
         proc = self._spawn_git(*args, **kwargs)
         stdout, stderr = proc.communicate()
+        self._close_proc_pipes(proc)
         self.assertEqual(0, proc.returncode,
                          'git exits with %r, args %r, kwargs %r, stdout %r, '
                          'stderr %r' %
@@ -72,6 +73,7 @@ class GitCommandMixin(object):
             data = data.encode('utf-8')
         proc = self._spawn_git('fast-import', stdin=PIPE)
         stdout, stderr = proc.communicate(input=data)
+        self._close_proc_pipes(proc)
         self.assertEqual(0, proc.returncode,
                          'git exits with %r, stdout %r, stderr %r' %
                          (proc.returncode, stdout, stderr))
@@ -93,6 +95,11 @@ class GitCommandMixin(object):
         env['GIT_COMMITTER_DATE'] = dt
         env['GIT_AUTHOR_DATE'] = dt
 
+    def _close_proc_pipes(self, proc):
+        for f in (proc.stdin, proc.stdout, proc.stderr):
+            if f:
+                f.close()
+
 
 class BaseTestCase(unittest.TestCase, GitCommandMixin):
 
@@ -251,12 +258,6 @@ class HistoryTimeRangeTestCase(BaseTestCase):
 
 class GitNormalTestCase(BaseTestCase):
 
-    def _create_req(self, **kwargs):
-        data = dict(args={}, perm=MockPerm(), href=Href('/'), chrome={},
-                    authname='trac', tz=utc, get_header=lambda name: None)
-        data.update(kwargs)
-        return Mock(**data)
-
     def test_get_node(self):
         self.env.config.set('git', 'persistent_cache', 'false')
         self.env.config.set('git', 'cached_repository', 'false')
@@ -314,20 +315,24 @@ class GitNormalTestCase(BaseTestCase):
         self.assertEqual(None, repos.oldest_rev)
         self.assertEqual(None, repos.normalize_rev(''))
         self.assertEqual(None, repos.normalize_rev(None))
+        self.assertEqual(None, repos.display_rev(''))
+        self.assertEqual(None, repos.display_rev(None))
+        self.assertEqual(None, repos.short_rev(''))
+        self.assertEqual(None, repos.short_rev(None))
 
         node = repos.get_node('/', youngest_rev)
         self.assertEqual([], list(node.get_entries()))
         self.assertEqual([], list(node.get_history()))
         self.assertRaises(NoSuchNode, repos.get_node, '/path', youngest_rev)
 
-        req = self._create_req(path_info='/browser/gitrepos')
+        req = MockRequest(self.env, path_info='/browser/gitrepos')
         browser_mod = BrowserModule(self.env)
         self.assertTrue(browser_mod.match_request(req))
         rv = browser_mod.process_request(req)
         self.assertEqual('browser.html', rv[0])
         self.assertEqual(None, rv[1]['rev'])
 
-        req = self._create_req(path_info='/log/gitrepos')
+        req = MockRequest(self.env, path_info='/log/gitrepos')
         log_mod = LogModule(self.env)
         self.assertTrue(log_mod.match_request(req))
         rv = log_mod.process_request(req)
@@ -364,7 +369,8 @@ class GitRepositoryTestCase(BaseTestCase):
 
     def test_invalid_path_raises(self):
         self.assertRaises(InvalidRepository, GitRepository, self.env,
-                          '/the/invalid/path', [], self.env.log)
+                          '/the/invalid/path', {'name': 'therepos'},
+                          self.env.log)
 
     def test_repository_instance(self):
         self._git_init()
@@ -496,6 +502,348 @@ class GitRepositoryTestCase(BaseTestCase):
                                                       rev))),
                          sorted(cset.get_changes()))
 
+    _data_annotations = """\
+blob
+mark :1
+data 14
+one
+two
+three
+
+reset refs/heads/master
+commit refs/heads/master
+mark :2
+author Joe <joe@example.com> 1467172510 +0000
+committer Joe <joe@example.com> 1467172510 +0000
+data 6
+blame
+M 100644 :1 test.txt
+
+blob
+mark :3
+data 49
+one
+two
+three
+four
+five
+six
+seven
+eight
+nine
+ten
+
+commit refs/heads/master
+mark :4
+author Joe <joe@example.com> 1467172511 +0000
+committer Joe <joe@example.com> 1467172511 +0000
+data 10
+add lines
+from :2
+M 100644 :3 test.txt
+
+blob
+mark :5
+data 40
+one
+two
+3
+four
+five
+6
+seven
+eight
+9
+ten
+
+commit refs/heads/master
+mark :6
+author Joe <joe@example.com> 1467172512 +0000
+committer Joe <joe@example.com> 1467172512 +0000
+data 13
+modify lines
+from :4
+M 100644 :5 test.txt
+
+reset refs/heads/master
+from :6
+
+"""
+
+    def test_get_annotations(self):
+        self._git_init(data=False)
+        self._git_fast_import(self._data_annotations)
+        self._add_repository('gitrepos')
+        repos = self._repomgr.get_repository('gitrepos')
+        repos.sync()
+
+        rev1 = 'a7efe353630d02139f255220d71b76fa68eb7132'  # root commit
+        rev2 = 'f928d1b36b8bedf64bcf08667428fdcccf36b21b'
+        rev3 = '279a097f111c7cb1ef0b9da39735188051fd4f69'  # HEAD
+
+        self.assertEqual([rev1] * 3,
+                         repos.get_node('test.txt', rev1).get_annotations())
+        self.assertEqual([rev1] * 3 + [rev2] * 7,
+                         repos.get_node('test.txt', rev2).get_annotations())
+
+        expected = [rev1, rev1, rev3, rev2, rev2, rev3, rev2, rev2, rev3, rev2]
+        self.assertEqual(expected,
+                         repos.get_node('test.txt', rev3).get_annotations())
+        self.assertEqual(expected,
+                         repos.get_node('test.txt', 'HEAD').get_annotations())
+        self.assertEqual(expected,
+                         repos.get_node('test.txt').get_annotations())
+
+    # *   79dff4ccf842f8e2d2da2ee3e7a2149df63b099b Merge branch 'A'
+    # |\
+    # | *   86387120095e9e43573bce61b9da70a8c5d1c1b9 Merge branch 'B' into A
+    # | |\
+    # | | * 64e12f96b6b3040cd9edc225734ab2b26a03758b Changed a1
+    # | * | 67fdcf11e2d083b123b9a79be4fce0600f313f81 Changed a2
+    # * | | 42fbe758709b2a65aba33e56b2f53cd126c190e3 Changed b2
+    # | |/
+    # |/|
+    # * | 24d94dc08eb77438e4ead192b3f7d1c7bdf1a9e1 Changed b2
+    # * | 998bf23843c8fd982bbc23f88ec33c4d08114557 Changed b1
+    # |/
+    # * c5b01c74e125aa034a1d4ae31dc16f1897a73779 First commit
+    _data_iter_nodes = """\
+blob
+mark :1
+data 2
+a1
+blob
+mark :2
+data 2
+a2
+blob
+mark :3
+data 2
+b1
+blob
+mark :4
+data 2
+b2
+reset refs/heads/A
+commit refs/heads/A
+mark :5
+author Joe <joe@example.com> 1470744252 +0000
+committer Joe <joe@example.com> 1470744252 +0000
+data 13
+First commit
+M 100644 :1 A/a1.txt
+M 100644 :2 A/a2.txt
+M 100644 :3 B/b1.txt
+M 100644 :4 B/b2.txt
+
+blob
+mark :6
+data 4
+b1-1
+commit refs/heads/master
+mark :7
+author Joe <joe@example.com> 1470744253 +0000
+committer Joe <joe@example.com> 1470744253 +0000
+data 11
+Changed b1
+from :5
+M 100644 :6 B/b1.txt
+
+blob
+mark :8
+data 4
+b2-1
+commit refs/heads/master
+mark :9
+author Joe <joe@example.com> 1470744254 +0000
+committer Joe <joe@example.com> 1470744254 +0000
+data 11
+Changed b2
+from :7
+M 100644 :8 B/b2.txt
+
+blob
+mark :10
+data 4
+b2-2
+commit refs/heads/master
+mark :11
+author Joe <joe@example.com> 1470744255 +0000
+committer Joe <joe@example.com> 1470744255 +0000
+data 11
+Changed b2
+from :9
+M 100644 :10 B/b2.txt
+
+blob
+mark :12
+data 4
+a2-1
+commit refs/heads/A
+mark :13
+author Joe <joe@example.com> 1470744256 +0000
+committer Joe <joe@example.com> 1470744256 +0000
+data 11
+Changed a2
+from :5
+M 100644 :12 A/a2.txt
+
+blob
+mark :14
+data 4
+a1-1
+commit refs/heads/B
+mark :15
+author Joe <joe@example.com> 1470744257 +0000
+committer Joe <joe@example.com> 1470744257 +0000
+data 11
+Changed a1
+from :9
+M 100644 :14 A/a1.txt
+
+commit refs/heads/A
+mark :16
+author Joe <joe@example.com> 1470744258 +0000
+committer Joe <joe@example.com> 1470744258 +0000
+data 24
+Merge branch 'B' into A
+from :13
+merge :15
+M 100644 :14 A/a1.txt
+M 100644 :6 B/b1.txt
+M 100644 :8 B/b2.txt
+
+commit refs/heads/master
+mark :17
+author Joe <joe@example.com> 1470744259 +0000
+committer Joe <joe@example.com> 1470744259 +0000
+data 17
+Merge branch 'A'
+from :11
+merge :16
+M 100644 :14 A/a1.txt
+M 100644 :12 A/a2.txt
+
+reset refs/heads/master
+from :17
+
+"""
+
+    def test_iter_nodes(self):
+        self._git_init(data=False)
+        self._git_fast_import(self._data_iter_nodes)
+        self._add_repository('gitrepos')
+        repos = self._repomgr.get_repository('gitrepos')
+        repos.sync()
+        mod = BrowserModule(self.env)
+
+        root_node = repos.get_node('')
+        nodes = list(mod._iter_nodes(root_node))
+        self.assertEqual(['79dff4ccf842f8e2d2da2ee3e7a2149df63b099b'] * 7,
+                         [node.rev for node in nodes])
+        self.assertEqual([
+            ('79dff4ccf842f8e2d2da2ee3e7a2149df63b099b', ''),
+            ('64e12f96b6b3040cd9edc225734ab2b26a03758b', 'A'),
+            ('64e12f96b6b3040cd9edc225734ab2b26a03758b', 'A/a1.txt'),
+            ('67fdcf11e2d083b123b9a79be4fce0600f313f81', 'A/a2.txt'),
+            ('42fbe758709b2a65aba33e56b2f53cd126c190e3', 'B'),
+            ('998bf23843c8fd982bbc23f88ec33c4d08114557', 'B/b1.txt'),
+            ('42fbe758709b2a65aba33e56b2f53cd126c190e3', 'B/b2.txt'),
+            ], [(node.created_rev, node.path) for node in nodes])
+
+        root_node = repos.get_node('',
+                                   '86387120095e9e43573bce61b9da70a8c5d1c1b9')
+        nodes = list(mod._iter_nodes(root_node))
+        self.assertEqual(['86387120095e9e43573bce61b9da70a8c5d1c1b9'] * 7,
+                         [node.rev for node in nodes])
+        self.assertEqual([
+            ('86387120095e9e43573bce61b9da70a8c5d1c1b9', ''),
+            ('64e12f96b6b3040cd9edc225734ab2b26a03758b', 'A'),
+            ('64e12f96b6b3040cd9edc225734ab2b26a03758b', 'A/a1.txt'),
+            ('67fdcf11e2d083b123b9a79be4fce0600f313f81', 'A/a2.txt'),
+            ('24d94dc08eb77438e4ead192b3f7d1c7bdf1a9e1', 'B'),
+            ('998bf23843c8fd982bbc23f88ec33c4d08114557', 'B/b1.txt'),
+            ('24d94dc08eb77438e4ead192b3f7d1c7bdf1a9e1', 'B/b2.txt'),
+            ], [(node.created_rev, node.path) for node in nodes])
+
+        root_commit = 'c5b01c74e125aa034a1d4ae31dc16f1897a73779'
+        root_node = repos.get_node('', root_commit)
+        nodes = list(mod._iter_nodes(root_node))
+        self.assertEqual([root_commit] * 7, [node.rev for node in nodes])
+        self.assertEqual([
+            (root_commit, ''),
+            (root_commit, 'A'),
+            (root_commit, 'A/a1.txt'),
+            (root_commit, 'A/a2.txt'),
+            (root_commit, 'B'),
+            (root_commit, 'B/b1.txt'),
+            (root_commit, 'B/b2.txt'),
+            ], [(node.created_rev, node.path) for node in nodes])
+
+    def test_colon_character_in_filename(self):
+        self._git_init(data=False)
+        self._git_fast_import(self._data_colon_character_in_filename)
+        self._add_repository('gitrepos')
+        repos = self._repomgr.get_repository('gitrepos')
+        repos.sync()
+        rev1 = '382e1e6b85ba20ce8a84af1a875eaa50b8e1e092'  # root commit
+        rev2 = 'd8001832aad079f85a39a54a388a8b15fe31093d'
+        ADD = Changeset.ADD
+        MOVE = Changeset.MOVE
+        FILE = Node.FILE
+
+        cset = repos.get_changeset(rev1)
+        self.assertEqual(set([('0100644',     FILE, ADD, None, None),
+                              ('0100644.txt', FILE, ADD, None, None),
+                              (':100644',     FILE, ADD, None, None),
+                              (':100644.txt', FILE, ADD, None, None),
+                              ('a100644',     FILE, ADD, None, None),
+                              ('a100644.txt', FILE, ADD, None, None)
+                             ]),
+                         set(cset.get_changes()))
+
+        cset = repos.get_changeset(rev2)
+        self.assertEqual(set([(':100666', FILE, MOVE, ':100644', rev1)]),
+                         set(cset.get_changes()))
+
+    _data_colon_character_in_filename = """\
+blob
+mark :1
+data 0
+
+blob
+mark :2
+data 16
+...............
+
+reset refs/heads/master
+commit refs/heads/master
+mark :3
+author Joe <joe@example.com> 1491387182 +0000
+committer Joe <joe@example.com> 1491387182 +0000
+data 9
+(#12758)
+M 100644 :1 0100644.txt
+M 100644 :1 0100644
+M 100644 :1 :100644.txt
+M 100644 :2 :100644
+M 100644 :1 a100644.txt
+M 100644 :1 a100644
+
+commit refs/heads/master
+mark :4
+author Joe <joe@example.com> 1491387183 +0000
+committer Joe <joe@example.com> 1491387183 +0000
+data 16
+(#12758) rename
+from :3
+D :100644
+M 100644 :2 :100666
+
+reset refs/heads/master
+from :4
+"""
+
     def _get_quickjump_names(self, repos):
         return list(name for type, name, path, rev
                          in repos.get_quickjump_entries('HEAD'))
@@ -544,6 +892,38 @@ class GitCachedRepositoryTestCase(GitRepositoryTestCase):
             repos.sync(feedback=feedback_2)  # restart sync
         self.assertEqual(revs, revs2)
 
+    def test_sync_file_with_invalid_byte_sequence(self):
+        self._git_init(data=False)
+        self._git_fast_import("""\
+blob
+mark :1
+data 0
+
+reset refs/heads/master
+commit refs/heads/master
+mark :2
+author Ryan Ollos <rjollos@edgewall.com> 1463639119 +0200
+committer Ryan Ollos <rjollos@edgewall.com> 1463639119 +0200
+data 9
+(#12322)
+M 100644 :1 "\312\326\267\347\307\331.txt"
+
+reset refs/heads/master
+from :2
+
+""")
+        self._add_repository('gitrepos')
+        repos = self._repomgr.get_repository('gitrepos')
+
+        revs = []
+        def feedback(rev):
+            revs.append(rev)
+        repos.sync(feedback=feedback)
+
+        changes = list(repos.repos.get_changeset(revs[0]).get_changes())
+        self.assertEqual(1, len(changes))
+        self.assertEqual(u'\ufffd\u05b7\ufffd\ufffd\ufffd.txt', changes[0][0])
+
     def test_sync_merge(self):
         self._git_init()
         self._create_merge_commit()
@@ -649,11 +1029,84 @@ M 100644 :1 dev%(dev)08d.txt
         return data.getvalue()
 
 
+class GitwebProjectsRepositoryProviderTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+        self.projects_base = os.path.realpath(tempfile.mkdtemp())
+        self.projects_list = os.path.join(self.projects_base, 'projects_list')
+        with open(self.projects_list, 'w') as f:
+            f.write("""
+            repos1 user1
+            repos2.git user+2+<user@example.com>
+
+            repos3
+            """)
+        self.env.config.set('gitweb-repositories', 'projects_list',
+                            self.projects_list)
+        self.env.config.set('gitweb-repositories', 'projects_base',
+                            self.projects_base)
+        self.env.config.set('gitweb-repositories', 'projects_url',
+                            'https://example.com/%s')
+
+    def tearDown(self):
+        self.env.shutdown()
+        rmtree(self.projects_base)
+
+    def test_project_list_path_not_found(self):
+        """Warning is logged when projects_list file is not found, but
+        exception is not raised.
+        """
+        os.remove(self.projects_list)
+        provider = GitwebProjectsRepositoryProvider(self.env)
+        repositories = list(provider.get_repositories())
+
+        self.assertEqual([], repositories)
+
+    def test_get_repositories(self):
+        provider = GitwebProjectsRepositoryProvider(self.env)
+        repositories = list(provider.get_repositories())
+
+        self.assertEqual(3, len(repositories))
+        self.assertEqual('repos1', repositories[0][0])
+        self.assertEqual('git', repositories[0][1]['type'])
+        self.assertEqual(os.path.join(self.projects_base, 'repos1'),
+                         repositories[0][1]['dir'])
+        self.assertEqual('https://example.com/repos1',
+                         repositories[0][1]['url'])
+        self.assertEqual('repos2', repositories[1][0])
+        self.assertEqual('git', repositories[1][1]['type'])
+        self.assertEqual(os.path.join(self.projects_base, 'repos2.git'),
+                         repositories[1][1]['dir'])
+        self.assertEqual('https://example.com/repos2',
+                         repositories[1][1]['url'])
+        self.assertEqual('repos3', repositories[2][0])
+        self.assertEqual('git', repositories[2][1]['type'])
+        self.assertEqual(os.path.join(self.projects_base, 'repos3'),
+                         repositories[2][1]['dir'])
+        self.assertEqual('https://example.com/repos3',
+                         repositories[2][1]['url'])
+
+
+
 class StopSync(Exception):
     pass
 
 
-def suite():
+class GitConnectorTestCase(BaseTestCase):
+
+    def _git_version_from_system_info(self):
+        git_version = None
+        for name, version in self.env.get_systeminfo():
+            if name == 'GIT':
+                git_version = version
+        return git_version
+
+    def test_get_system_info(self):
+        self.assertIsNotNone(self._git_version_from_system_info())
+
+
+def test_suite():
     suite = unittest.TestSuite()
     if GitCommandMixin.git_bin:
         suite.addTest(unittest.makeSuite(SanityCheckingTestCase))
@@ -662,6 +1115,8 @@ def suite():
         suite.addTest(unittest.makeSuite(GitNormalTestCase))
         suite.addTest(unittest.makeSuite(GitRepositoryTestCase))
         suite.addTest(unittest.makeSuite(GitCachedRepositoryTestCase))
+        suite.addTest(unittest.makeSuite(GitConnectorTestCase))
+        suite.addTest(unittest.makeSuite(GitwebProjectsRepositoryProviderTestCase))
     else:
         print("SKIP: tracopt/versioncontrol/git/tests/git_fs.py (git cli "
               "binary, 'git', not found)")
@@ -669,4 +1124,4 @@ def suite():
 
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
